@@ -1,6 +1,7 @@
 import asyncio
 import logging
 import multiprocessing
+import os
 import threading
 from concurrent.futures import ThreadPoolExecutor
 from typing import Any, Callable, Dict, List, Literal, Optional, Tuple
@@ -39,6 +40,7 @@ class Executor:
         update_routes: Dict[str, List[Route]],
         update_task_type: Literal["thread", "process"] = "thread",
         disable_update_task: bool = False,
+        redis_socket_timeout: int = 60,
     ):
         self._instance_name = instance_name
         self._cache_ttl = cache_ttl
@@ -47,8 +49,30 @@ class Executor:
         self._init_state_params = init_state_params
         self._load_state_func = load_state_func
         self._save_state_func = save_state_func
+        self.__lock_prefix = (
+            f"MOTION_LOCK:DEV:{self._instance_name}"
+            if os.getenv("MOTION_ENV", "dev") == "dev"
+            else f"MOTION_LOCK:{self._instance_name}"
+        )
+        self.__queue_prefix = (
+            f"MOTION_QUEUE:DEV:{self._instance_name}"
+            if os.getenv("MOTION_ENV", "dev") == "dev"
+            else f"MOTION_QUEUE:{self._instance_name}"
+        )
+        self.__channel_prefix = (
+            f"MOTION_CHANNEL:DEV:{self._instance_name}"
+            if os.getenv("MOTION_ENV", "dev") == "dev"
+            else f"MOTION_CHANNEL:{self._instance_name}"
+        )
+        self.__cache_result_prefix = (
+            f"MOTION_RESULT:DEV:{self._instance_name}"
+            if os.getenv("MOTION_ENV", "dev") == "dev"
+            else f"MOTION_RESULT:{self._instance_name}"
+        )
 
         self.running: Any = multiprocessing.Value("b", False)
+        self._redis_socket_timeout = redis_socket_timeout
+
         self._redis_params, self._redis_con = self._connectToRedis()
         try:
             self._redis_con.ping()
@@ -62,28 +86,28 @@ class Executor:
         self.running.value = True
 
         # Set up state
-        self.version = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
-        self._state = State(
-            instance_name.split("__")[0],
-            instance_name.split("__")[1],
-            {},
+        state, version = loadState(
+            self._redis_con, self._instance_name, self._load_state_func
         )
-        if self.version is None:
-            self.version = 1
-            # Setup state
-            self._state.update(self.setUp(**self._init_state_params))
-            saveState(
-                self._state,
-                self._redis_con,
-                self._instance_name,
-                self._save_state_func,
-            )
-        else:
-            # Load state
-            self._state = self._loadState()
-            self.version = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
 
-        self.version = int(self.version)
+        # If state does not exist, run setUp
+        if state is None:
+            with self._redis_con.lock(self.__queue_prefix, timeout=120):
+                state = State(
+                    instance_name.split("__")[0], instance_name.split("__")[1], {}
+                )
+                state.update(self.setUp(**self._init_state_params))
+                version = saveState(
+                    state,
+                    0,
+                    self._redis_con,
+                    self._instance_name,
+                    self._save_state_func,
+                )
+                assert version == 1, "Version should be 1 after saving state."
+
+        self._state = state
+        self.version = version
 
         # Set up routes
         self._serve_routes: Dict[str, Route] = serve_routes
@@ -111,16 +135,61 @@ class Executor:
 
     def _connectToRedis(self) -> Tuple[RedisParams, redis.Redis]:
         rp = get_redis_params()
-        r = redis.Redis(
-            host=rp.host,
-            port=rp.port,
-            password=rp.password,
-            db=rp.db,
-        )
+
+        # Put a timeout on the connection
+        param_dict = rp.dict()
+        if "socket_timeout" not in param_dict:
+            param_dict["socket_timeout"] = self._redis_socket_timeout
+
+        # Pop all None values
+        param_dict = {k: v for k, v in param_dict.items() if v is not None}
+
+        r = redis.Redis(**param_dict)
         return rp, r
 
-    def _loadState(self) -> State:
-        return loadState(self._redis_con, self._instance_name, self._load_state_func)
+    def _loadState(self) -> None:
+        # If in dev mode, try loading dev state
+        redis_v = None
+        if os.getenv("MOTION_ENV", "dev") == "dev":
+            redis_v = self._redis_con.get(f"MOTION_VERSION:DEV:{self._instance_name}")
+
+        if not redis_v:
+            redis_v = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
+        if not redis_v:
+            raise ValueError(
+                f"Error loading state for {self._instance_name}." + " No version found."
+            )
+
+        if self.version and self.version < int(redis_v):
+            # Reload state
+            new_state, self.version = loadState(
+                self._redis_con, self._instance_name, self._load_state_func
+            )
+            if new_state is None:
+                raise ValueError(
+                    f"Error loading state for {self._instance_name}."
+                    + " State is None."
+                )
+            self._state = new_state
+
+    def _saveState(self, new_state: State) -> None:
+        # Save state to redis
+        new_version = saveState(
+            new_state,
+            self.version,
+            self._redis_con,
+            self._instance_name,
+            self._save_state_func,
+        )
+        if new_version == -1:
+            logger.error(
+                f"Error saving state to Redis for {self._instance_name}:"
+                + " there was a newer state found."
+            )
+            # Reload state
+            self._loadState()
+        else:
+            self.version = new_version
 
     def setUp(self, **kwargs: Any) -> Dict[str, Any]:
         # Set up initial state
@@ -161,10 +230,8 @@ class Executor:
                 load_state_func=self._load_state_func,
                 queue_identifiers=self.queue_ids_for_fit,
                 channel_identifiers=self.channel_dict_for_fit,
-                redis_host=self._redis_params.host,
-                redis_port=self._redis_params.port,
-                redis_db=self._redis_params.db,
-                redis_password=self._redis_params.password,  # type: ignore
+                lock_identifier=self.__lock_prefix,
+                redis_params=self._redis_params.dict(),
                 running=self.running,
             )
             self.worker_task.start()  # type: ignore
@@ -200,10 +267,8 @@ class Executor:
                     load_state_func=self._load_state_func,
                     queue_identifiers=self.queue_ids_for_fit,
                     channel_identifiers=self.channel_dict_for_fit,
-                    redis_host=self._redis_params.host,
-                    redis_port=self._redis_params.port,
-                    redis_db=self._redis_params.db,
-                    redis_password=self._redis_params.password,  # type: ignore
+                    lock_identifier=self.__lock_prefix,
+                    redis_params=self._redis_params.dict(),
                     running=self.running,
                 )
                 self.worker_task.start()  # type: ignore
@@ -216,11 +281,11 @@ class Executor:
 
     def _get_queue_identifier(self, route_key: str, udf_name: str) -> str:
         """Gets the queue identifier for a given route key and UDF name."""
-        return f"MOTION_QUEUE:{self._instance_name}/{route_key}/{udf_name}"
+        return f"{self.__queue_prefix}/{route_key}/{udf_name}"
 
     def _get_channel_identifier(self, route_key: str, udf_name: str) -> str:
         """Gets the channel identifier for a given route key and UDF name."""
-        return f"MOTION_CHANNEL:{self._instance_name}/{route_key}/{udf_name}"
+        return f"{self.__channel_prefix}/{route_key}/{udf_name}"
 
     def shutdown(self, is_open: bool) -> None:
         if self.disable_update_task:
@@ -271,43 +336,21 @@ class Executor:
 
         # Get latest state
         if use_lock:
-            with self._redis_con.lock(
-                f"MOTION_LOCK:{self._instance_name}", timeout=120
-            ):
+            with self._redis_con.lock(self.__lock_prefix, timeout=120):
                 if force_update:
-                    self._state = self._loadState()
+                    self._loadState()
                 self._state.update(new_state)
 
                 # Save state to redis
-                saveState(
-                    self._state,
-                    self._redis_con,
-                    self._instance_name,
-                    self._save_state_func,
-                )
-
-                version = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
-                if version is None:
-                    raise ValueError("Version not found in Redis.")
-                self.version = int(version)
+                self._saveState(self._state)
 
         else:
             if force_update:
-                self._state = self._loadState()
+                self._loadState()
             self._state.update(new_state)
 
             # Save state to redis
-            saveState(
-                self._state,
-                self._redis_con,
-                self._instance_name,
-                self._save_state_func,
-            )
-
-            version = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
-            if version is None:
-                raise ValueError("Version not found in Redis.")
-            self.version = int(version)
+            self._saveState(self._state)
 
     def _enqueue_and_trigger_update(
         self,
@@ -329,11 +372,9 @@ class Executor:
 
                     # Hold lock
 
-                    with self._redis_con.lock(
-                        f"MOTION_LOCK:{self._instance_name}", timeout=120
-                    ):
+                    with self._redis_con.lock(self.__lock_prefix, timeout=120):
                         try:
-                            self._state = self._loadState()
+                            self._loadState()
 
                             state_update = route.run(
                                 state=self._state,
@@ -399,11 +440,9 @@ class Executor:
                 if flush_update:
                     route = self._update_routes[key][update_udf_name]
 
-                    with self._redis_con.lock(
-                        f"MOTION_LOCK:{self._instance_name}", timeout=120
-                    ):
+                    with self._redis_con.lock(self.__lock_prefix, timeout=120):
                         try:
-                            self._state = self._loadState()
+                            self._loadState()
 
                             state_update = route.run(
                                 state=self._state,
@@ -465,14 +504,7 @@ class Executor:
         serve_result = None
 
         if force_refresh:
-            self._state = self._loadState()
-            v = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
-            if not v:
-                raise ValueError(
-                    f"Error loading state for {self._instance_name}."
-                    + " No version found."
-                )
-            self.version = int(v)
+            self._loadState()
 
         # If caching is disabled, return
         if self._cache_ttl == 0:
@@ -487,11 +519,13 @@ class Executor:
         # Check if key is in cache if value can be hashed and
         # user doesn't want to force refresh state
         if value_hash and not force_refresh and not ignore_cache:
-            cache_result_key = f"MOTION_RESULT:{self._instance_name}/{key}/{value_hash}"
+            cache_result_key = f"{self.__cache_result_prefix}/{key}/{value_hash}"
             if self._redis_con.exists(cache_result_key):
-                props = cloudpickle.loads(self._redis_con.get(cache_result_key))
-                serve_result = props.serve_result
-                route_run = True
+                new_props = cloudpickle.loads(self._redis_con.get(cache_result_key))
+                if new_props._serve_result is not None:
+                    props = new_props
+                    serve_result = props.serve_result
+                    route_run = True
 
         return route_run, serve_result, props, value_hash
 
@@ -535,7 +569,7 @@ class Executor:
                 # Cache result
                 if value_hash:
                     cache_result_key = (
-                        f"MOTION_RESULT:{self._instance_name}/{key}/{value_hash}"
+                        f"{self.__cache_result_prefix}/{key}/{value_hash}"
                     )
                     self.tp.submit(self._setRedis, cache_result_key, props)
 
@@ -546,7 +580,9 @@ class Executor:
         )
 
         if not route_hit:
-            raise KeyError(f"Key {key} not in routes.")
+            raise KeyError(
+                f"Key {key} not in routes for component {self._instance_name}."
+            )
 
         return serve_result
 
@@ -597,7 +633,9 @@ class Executor:
         )
 
         if not route_hit:
-            raise KeyError(f"Key {key} not in routes.")
+            raise KeyError(
+                f"Key {key} not in routes for component {self._instance_name}."
+            )
 
         return serve_result
 
@@ -637,11 +675,4 @@ class Executor:
         # Wait for update result to finish
         update_events.wait()
         # Update state
-        self._state = self._loadState()
-        v = self._redis_con.get(f"MOTION_VERSION:{self._instance_name}")
-        if not v:
-            raise ValueError(
-                f"Error loading state for {self._instance_name}." + " No version found."
-            )
-
-        self.version = int(v)
+        self._loadState()
